#version 120
/////////ADJUSTABLE VARIABLES//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////ADJUSTABLE VARIABLES//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define HDR

#define TONEMAP 4 //[1 2 3 4 5]

	/////////INTERNAL VARIABLES////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////INTERNAL VARIABLES////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

uniform sampler2D colortex0;

varying vec2 texcoord;

/////////////////////////FUNCTIONS/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////FUNCTIONS//////////////////////////////////////////////////////////////////////////////////

vec3 BetterColors(in vec3 color) {
    vec3 BetterColoredImage;

    vec3 overExposed = color * 1.0;

    vec3 underExposed = color / 1.0;

    BetterColoredImage = mix(underExposed, overExposed, color);


    return BetterColoredImage;
}

vec3 BOTWTonemap(vec3 color){
    color = pow(color, vec3(1.0 / 1.2));

    float avg = (color.r + color.g + color.b) / 5.0;
    float maxc = max(color.r, max(color.g, color.b));

    float w = 1.0 - pow(1.0 - 1.0 * avg, 2.0);
    float weight = 1.0 + w * 0.18;

    return mix(vec3(maxc), color * 1.0, weight);
}

vec3 BWTonemap(vec3 color){
    color = pow(color, vec3(1.0 / 1.2));

    float avg = (color.r + color.g + color.b) / 5.0;
    float maxc = max(color.r, max(color.g, color.b));

    float w = 1.0 - pow(1.0 - 1.0 * avg, 2.0);
    float weight = 0.0 + w * 0.0;

    return mix(vec3(maxc), color * 1.0, weight);
}

vec3 VibrantTonemap(vec3 color){
    color = pow(color, vec3(1.0 / 1.2));

    float avg = (color.r + color.g + color.b) / 5.0;
    float maxc = max(color.r, max(color.g, color.b));

    float w = 1.0 - pow(1.0 - 1.0 * avg, 2.0);
    float weight = 1.5 + w * 0.18;

    return mix(vec3(maxc), color * 1.0, weight);
}

vec3 NegativeTonemap(vec3 color){
    color = pow(color, vec3(BetterColors(color) * 5.0));

    float avg = (color.r + color.g + color.b) / 5.0;
    float maxc = max(color.r, max(color.g, color.b));

    float w = 1.0 - pow(1.0 - 1.0 * avg, 2.0);
    float weight = 1.0 + w * 0.18;

    return mix(vec3(maxc), color * 1.0, weight);
}

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////MAIN//////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void main() {
	vec3 color = texture2D(colortex0, texcoord).rgb;

#ifdef HDR
	color = BetterColors(color);
#endif

#if TONEMAP == 1
  color = color;
#endif

#if TONEMAP == 2
  color = BOTWTonemap(color);
#endif

#if TONEMAP == 3
  color = BWTonemap(color);
#endif

#if TONEMAP == 4
  color = VibrantTonemap(color);
#endif

#if TONEMAP == 5
  color = NegativeTonemap(color);
#endif

/* DRAWBUFFERS:0 */
	gl_FragData[0] = vec4(color, 1.0); //colortex0
}
